import os
from PyQt5.QtWidgets import QApplication, QWidget, QVBoxLayout, QPushButton, QGridLayout, QMessageBox, QSpacerItem, QSizePolicy, QFrame, QSpinBox, QLabel, QHBoxLayout
from PyQt5.QtGui import QPixmap, QPalette, QBrush
from PyQt5.QtCore import Qt, pyqtSignal
from configuracio.gestio_config import load_config, save_config
from components.habitacio import Habitacio
from components.configuracio import Configuracio
from components.pantalla_inici import PantallaInici
import paho.mqtt.client as mqtt
from configuracio.constants import CONFIG_FILE, MQTT_BROKER, MQTT_PORT, HABITACIONS, assets_dir, BASE_ASSETS_DIR



class AplicacioPrincipal(QWidget):
    """
    Classe principal que gestiona la interfície gràfica de l'aplicació.
    Inclou la configuració, monitorització de dades en temps real, i integració amb MQTT.
    """

    actualitzar_signal = pyqtSignal(str, str, float)  # Signal per actualitzar la GUI amb el topic, nom habitació, i valor

    def __init__(self):
        """
        Inicialitza la classe principal carregant la configuració, definint el nombre d'habitacions,
        inicialitzant la interfície gràfica i establint la connexió amb MQTT.
        """

        super().__init__()
        self.config = load_config()  # Carrega la configuració des del fitxer
        self.num_habitacions_actual = self.config.get("num_habitacions", 9)  # Llegeix el nombre d'habitacions guardat
        self.habitacions = {}
        self.initUI()  # Inicialitza la interfície gràfica
        self.client = self.connect_mqtt()  # Connecta amb el broker MQTT

    def initUI(self):
        """
        Crea i configura la interfície gràfica principal.
        """

        self.layout = QVBoxLayout()
        self.layout.setContentsMargins(0, 0, 0, 0)  
        fondo_path = os.path.join(BASE_ASSETS_DIR, "fondo.png")
        fondo = QPixmap(fondo_path)
        palette = QPalette()
        palette.setBrush(QPalette.Background, QBrush(fondo))
        self.setPalette(palette)
        #self.setStyleSheet("background-color: #f5f5dc;")

        #6DD8A5
        # Contenidor per la barra superior amb color de fons
        self.barra_superior_widget = QWidget()
        self.barra_superior_widget.setStyleSheet("""
        background-color: #6DD8A5;
        border-bottom: 1px solid #000000;  
    """)
        self.barra_superior_widget.setFixedHeight(55)  
        self.barra_superior = QHBoxLayout(self.barra_superior_widget)
        self.barra_superior.setContentsMargins(0, 0, 0, 0)  

        # Icona del programa
        self.icon_label = QLabel()
        icon_path = os.path.join(BASE_ASSETS_DIR, "icono.png")
        icon_pixmap = QPixmap(icon_path).scaled(50, 50, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        self.icon_label.setPixmap(icon_pixmap)
        self.barra_superior.addWidget(self.icon_label, alignment=Qt.AlignLeft)

        # Títol dins la barra personalitzada
        self.titol_label = QLabel("Monitorització Geriàtric", self)
        self.titol_label.setStyleSheet("color: white; font-size: 18px; font-weight: bold;")
        self.barra_superior.addWidget(self.titol_label, alignment=Qt.AlignLeft)

        self.barra_superior.addStretch()

        # Botó de configuració dins la barra superior
        self.boto_configuracio = QPushButton("⚙ Configuració", self)
        self.boto_configuracio.setStyleSheet("""
            QPushButton {
                color: white;
                background-color: #37B47E;  /* Fons lleugerament opac */
                padding: 10px 20px;
                border: none;
                border-radius: 10px;
                font-size: 16px;
                font-family: 'Eurostile', sans-serif;
                font-weight: bold;
            }
            QPushButton:hover {
                background-color: rgba(255, 255, 255, 0.3);  /* Més opac en hover */
            }
        """)
        self.boto_configuracio.clicked.connect(self.obrir_configuracio)
        self.barra_superior.addWidget(self.boto_configuracio, alignment=Qt.AlignRight)

        # Botó de sortir dins la barra superior en color roig
        self.boto_sortir = QPushButton("Sortir", self)
        self.boto_sortir.setStyleSheet("""
            QPushButton {
                color: white;
                background-color: #FF6347;  /* Fons vermell opac */
                padding: 10px 20px;
                border: none;
                border-radius: 10px;
                font-size: 16px;
                font-family: 'Eurostile', sans-serif;
                font-weight: bold;
            }
            QPushButton:hover {
                background-color: rgba(255, 0, 0, 0.8);  /* Més opac en hover */
            }
        """)
        self.boto_sortir.clicked.connect(self.close)
        self.barra_superior.addWidget(self.boto_sortir, alignment=Qt.AlignRight)

        # Afegir la barra superior al layout principal
        self.layout.addWidget(self.barra_superior_widget)

        # Layout per les habitacions
        self.central_layout = QHBoxLayout()

        # mantenir el grid centrat verticalment
        spacer_superior = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)
        self.layout.addItem(spacer_superior)
            
        # Afegir espais a l'esquerra i dreta 
        spacer_left = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        spacer_right = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        
        self.central_layout.addItem(spacer_left)
        self.grid_layout = QGridLayout()
        self.central_layout.addLayout(self.grid_layout)
        self.central_layout.addItem(spacer_right)
        self.layout.addLayout(self.central_layout)
        spacer_inferior = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)
        self.layout.addItem(spacer_inferior)

        # Barra inferior 
        self.barra_inferior_widget = QWidget()
        self.barra_inferior_widget.setStyleSheet("""
        background-color: #6DD8A5;
        border-top: 1px solid #000000;  
    """)

        self.barra_inferior_widget.setFixedHeight(55) 
        self.barra_inferior_layout = QHBoxLayout(self.barra_inferior_widget)
        self.barra_inferior_layout.setContentsMargins(0, 0, 10, 0)  

        # Botó d'ajuda 
        self.boto_ajuda = QPushButton("?")
        self.boto_ajuda.setFixedSize(40, 40)  
        self.boto_ajuda.setStyleSheet("""
            QPushButton {
                background-color: #37B47E;  /* Verd */
                color: white;
                border: none;  
                border-radius: 20px;  
                font-size: 18px;
                font-weight: bold;
                 
            }
            QPushButton:hover {
                background-color: #2e9e6f;  
            }
            QPushButton:pressed {
                background-color: #26865f;  
            }
        """)
        self.boto_ajuda.clicked.connect(self.mostrar_informacio_ajuda)  # Connecta amb la funció que mostra la informació

        self.barra_inferior_layout.addStretch()  
        self.barra_inferior_layout.addWidget(self.boto_ajuda)
        self.layout.addWidget(self.barra_inferior_widget)

        self.setLayout(self.layout)

        # Inicialitzar habitacions
        self.habitacions = {}
        self.actualitzar_habitacions(self.num_habitacions_actual)  # Mostra el nombre correcte d'habitacions

        self.setWindowTitle('Monitorització Geriàtric')
        self.actualitzar_signal.connect(self.actualitzar_des_de_signal)  # Connectar el signal amb la funció d'actualització

    def mostrar_informacio_ajuda(self):
        """
        Mostra una finestra emergent amb instruccions per utilitzar el programa.
        """

        msg = QMessageBox()
        msg.setWindowTitle("Ajuda del Programa")
        msg.setText("Instruccions per utilitzar el programa:\n\n"
                    "1. Les habitacions mostren l'estat del pacient.\n"
                    "   - Llit verd: Temperatura i humitat normals.\n"
                    "   - Llit roig: Temperatura o humitat fora dels valors         normals.\n"
                    "   - Habitació verda: Cap caiguda detectada.\n"
                    "   - Habitació roja: Possible caiguda detectada.\n\n"
                    "2. Fes clic en el llit de l'habitació per veure més detalls.\n"
                    "3. Utilitza el botó de configuració per ajustar els valors normals de cada habitació.\n\n"
                    "Si tens qualsevol dubte, contacta amb el suport tècnic.")
        msg.setIcon(QMessageBox.Information)
        msg.setStandardButtons(QMessageBox.Ok)

        # estil de la finestra d'ajuda
        msg.setStyleSheet("""
            QMessageBox {
                background-color: #ffffff;
                font-family: 'Eurostile', sans-serif;
                font-size: 18px;
                color: #333333;
            }
            QPushButton {
                background-color: #37B47E;
                color: white;
                padding: 5px 10px;
                font-family: 'Eurostile', sans-serif;
                font-weight: bold;
                border-radius: 5px;
            }
            QPushButton:hover {
                background-color: #2e9e6f;
            }
            QPushButton:pressed {
                background-color: #26865f;
            }
        """)
        
        msg.exec_()


    def mostrar_logo(self):
        """
        Amaga la finestra principal i mostra la pantalla d'inici (logo) durant el llançament de l'aplicació.
        """

        self.hide()
        self.pantalla_inici = PantallaInici(self)
        self.pantalla_inici.show()


    def actualitzar_des_de_signal(self, habitacio, tipus, valor):
        """
        Actualitza la interfície gràfica segons un senyal rebut per MQTT.

        Paràmetres:
        ----------
        habitacio (str): Nom de l'habitació afectada.
        tipus (str): Tipus de dada rebuda ("temp", "hum", "alerta").
        valor (float): Valor associat a la dada (temperatura, humitat o indicador d'alerta).
        """

        if tipus == "temp":
            self.habitacions[habitacio].actualitzar_color_llit(temperatura=valor)
        elif tipus == "hum":
            self.habitacions[habitacio].actualitzar_color_llit(humitat=valor)
        elif tipus == "alerta":
            self.habitacions[habitacio].detectar_caiguda()

    def actualitzar_habitacions(self, num_habitacions):
        """
        Mostra el nombre d'habitacions especificat, ajustant el layout segons la configuració.

        Paràmetres:
        ----------
        num_habitacions (int): Nombre d'habitacions a mostrar.
        """

        self.num_habitacions_actual = num_habitacions
        self.grid_layout.setHorizontalSpacing(250)
        self.grid_layout.setVerticalSpacing(20)

        # Configurar posicions per a un màxim de 10 habitacions amb 3 columnes
        num_columnes = 3
        posicions = [(i, j) for i in range(4) for j in range(num_columnes)]  # 4 files x 3 columnes

        # Crear o actualitzar les habitacions segons la configuració
        for index in range(num_habitacions):
            nom_habitacio = f"Habitació {index + 1}"

            # Recuperar configuració de l'habitació
            habitacio_config = self.config["habitacions"].get(
                nom_habitacio,
                {"temp_min": 20, "temp_max": 30, "hum_min": 30, "hum_max": 60}
            )

            if nom_habitacio not in self.habitacions:
                # Crear una nova habitació si no existeix
                habitacio = Habitacio(
                    nom=nom_habitacio,
                    temp_min_norm=habitacio_config["temp_min"],
                    temp_max_norm=habitacio_config["temp_max"],
                    hum_min_norm=habitacio_config["hum_min"],
                    hum_max_norm=habitacio_config["hum_max"]
                )
                habitacio.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
                self.habitacions[nom_habitacio] = habitacio
                pos = posicions[index]
                self.grid_layout.addWidget(habitacio, pos[0], pos[1])
            else:
                # Actualitzar els valors de la configuració a l'habitació existent
                habitacio = self.habitacions[nom_habitacio]
                habitacio.temp_min_norm = habitacio_config["temp_min"]
                habitacio.temp_max_norm = habitacio_config["temp_max"]
                habitacio.hum_min_norm = habitacio_config["hum_min"]
                habitacio.hum_max_norm = habitacio_config["hum_max"]
                habitacio.show() 

        # Amagar les habitacions que estan per sobre del nombre seleccionat
        for index in range(num_habitacions, len(self.habitacions)):
            nom_habitacio = f"Habitació {index + 1}"
            if nom_habitacio in self.habitacions:
                self.habitacions[nom_habitacio].hide()

        # Ajustar les files i columnes per repartir les habitacions uniformement
        for i in range(4):  # Fins a 4 files
            self.grid_layout.setRowStretch(i, 1)
        for j in range(num_columnes):  # Fins a 3 columnes
            self.grid_layout.setColumnStretch(j, 1)

    def connect_mqtt(self):
        """
        Estableix la connexió amb el broker MQTT, subscrivint-se als temes de temperatura, humitat i alertes.

        Retorna:
        -------
        client: L'objecte MQTT client per gestionar les dades en temps real.
        """
        client = mqtt.Client()
        client.on_message = self.on_message
        client.connect(MQTT_BROKER, MQTT_PORT, 60)

        for topics in HABITACIONS.values():
            client.subscribe(topics["temp"])
            client.subscribe(topics["hum"])
            client.subscribe(topics["alerta"])

        client.loop_start()
        return client
        
    def obrir_configuracio(self):
        """
        Obre la finestra de configuració general i amaga la finestra principal mentre es mostra.
        """

        self.hide()
        # Crear una nova finestra de configuració si no existeix
        if not hasattr(self, 'configuracio_finestra') or self.configuracio_finestra is None:
            self.configuracio_finestra = Configuracio(self)
        else:
            # Actualitzar el spinbox abans de mostrar la finestra de configuració
            self.configuracio_finestra.spin_habitacions.setValue(self.num_habitacions_actual)
        # Mostrar la finestra de configuració
        self.configuracio_finestra.showMaximized()  

    

    def on_message(self, client, userdata, message):
        """
        Gestiona els missatges rebuts des del broker MQTT, actualitzant la interfície segons el topic i el valor.

        Paràmetres:
        ----------
        client: Objecte MQTT client.
        userdata: Informació de l'usuari 
        message: Missatge MQTT rebut amb el tema i contingut.
        """

        payload = message.payload.decode("utf-8")
        topic = message.topic

        # Actualitzar la interfície segons el topic rebut
        for habitacio, topics in HABITACIONS.items():
            if topic == topics["temp"]:
                temperatura = float(payload)
                self.actualitzar_signal.emit(habitacio, "temp", temperatura)  # Emetre la senyal per temperatura
            elif topic == topics["hum"]:
                humitat = float(payload)
                self.actualitzar_signal.emit(habitacio, "hum", humitat)  # Emetre la senyal per humitat
            elif topic == topics["alerta"] and payload == "Caiguda detectada":
                # Gestionar la caiguda detectada
                self.actualitzar_signal.emit(habitacio, "alerta", 0)  # Emetre la senyal per alerta
