from PyQt5.QtWidgets import QDialog, QVBoxLayout, QSlider, QLabel, QPushButton, QHBoxLayout
from PyQt5.QtCore import Qt
from configuracio.gestio_config import save_config, load_config


class ConfigHabitacio(QDialog):
    """
    Configura els valors específics d'una habitació individual (temperatura i humitat mínimes i màximes).
    """

    def __init__(self, app_principal, habitacio_nom, configuracio_pare):
        """
        Inicialitza la finestra de configuració individual d'una habitació.

        Paràmetres:
        ----------
        app_principal: Referència a l'objecte principal de l'aplicació.
        habitacio_nom (str): Nom de l'habitació a configurar.
        configuracio_pare: Referència a la finestra de configuració general.
        """
        super().__init__()
        self.app_principal = app_principal
        self.habitacio_nom = habitacio_nom
        self.configuracio_pare = configuracio_pare
        self.habitacio = app_principal.habitacions.get(habitacio_nom)
        self.initUI()

    def initUI(self):
        """
        Configura la interfície gràfica de la finestra, incloent sliders per modificar
        valors mínims i màxims de temperatura i humitat.
        """

        self.setWindowTitle(f"Configuració de {self.habitacio_nom}")
        self.setFixedWidth(400)  

        layout = QVBoxLayout()

        # sliders per a Min Temp, Max Temp, Min Hum i Max Hum
        self.temp_min_slider, self.temp_min_label = self.crear_slider(
        "Min Temp", 10, 40, self.habitacio.temp_min_norm, "temp_min"
        )
        self.temp_max_slider, self.temp_max_label = self.crear_slider(
            "Max Temp", 10, 40, self.habitacio.temp_max_norm, "temp_max"
        )
        self.hum_min_slider, self.hum_min_label = self.crear_slider(
            "Min Hum", 10, 90, self.habitacio.hum_min_norm, "hum_min"
        )
        self.hum_max_slider, self.hum_max_label = self.crear_slider(
            "Max Hum", 10, 90, self.habitacio.hum_max_norm, "hum_max"
        )
        layout.addWidget(self.temp_min_label)
        layout.addWidget(self.temp_min_slider)
        layout.addWidget(self.temp_max_label)
        layout.addWidget(self.temp_max_slider)
        layout.addWidget(self.hum_min_label)
        layout.addWidget(self.hum_min_slider)
        layout.addWidget(self.hum_max_label)
        layout.addWidget(self.hum_max_slider)

        # Botó per guardar els valors
        boto_guardar = QPushButton("Guardar")
        boto_guardar.setStyleSheet("""
            QPushButton {
                background-color: #37B47E;
                color: white;
                padding: 10px;
                font-size: 14px;
                font-family: 'Eurostile', sans-serif;
                font-weight: bold;
                border-radius: 5px;
            }
            QPushButton:hover {
                background-color: #2e9e6f;
            }
            QPushButton:pressed {
                background-color: #26865f;
            }
        """)
        boto_guardar.clicked.connect(self.guardar_valors)
        layout.addWidget(boto_guardar)

        self.setLayout(layout)

    def crear_slider(self, prefix, min_val, max_val, valor_actual, tipus):
        """
        Crea un slider amb etiqueta per ajustar un paràmetre específic de l'habitació.

        Paràmetres:
        ----------
        prefix (str): Etiqueta prefixada (e.g., "Min Temp").
        min_val (int): Valor mínim del slider.
        max_val (int): Valor màxim del slider.
        valor_actual (int): Valor inicial del slider.
        tipus (str): Tipus de valor a modificar (e.g., "temp_min").

        Retorna:
        -------
        tuple: Slider i etiqueta associada.
        """

        # Crear el slider
        slider = QSlider(Qt.Horizontal)
        slider.setRange(min_val, max_val)
        slider.setValue(valor_actual)

        # Crear l'etiqueta
        label = QLabel(f"{prefix}: {slider.value()}")

        # Actualitzar etiqueta i configuració al moure el slider
        def update_label(value):
            label.setText(f"{prefix}: {value}")
            setattr(self.habitacio, tipus, value)  # Actualitza el valor a l'objecte habitació
        
        slider.valueChanged.connect(update_label)

        # Estils del slider
        slider.setStyleSheet("""
            QSlider::groove:horizontal {
                height: 8px;
                background: #d3d3d3;
                border-radius: 4px;
            }
            QSlider::handle:horizontal {
                background: #37B47E;
                border: 1px solid #5c5c5c;
                width: 16px;
                height: 16px;
                margin: -5px 0;
                border-radius: 8px;
            }
            QSlider::sub-page:horizontal {
                background: #37B47E;
                border-radius: 4px;
            }
            QSlider::add-page:horizontal {
                background: #d3d3d3;
                border-radius: 4px;
            }
        """)

        # Layout per agrupar el slider i l'etiqueta
        layout = QHBoxLayout()
        layout.addWidget(label)
        layout.addWidget(slider)

        return slider, label

       


    def guardar_valors(self):
        """
        Guarda els valors configurats a la habitació i actualitza la configuració global.

        Accions:
        --------
        - Modifica els valors de temperatura i humitat mínims i màxims.
        - Actualitza la configuració general de l'aplicació.
        - Escriu els canvis al fitxer de configuració.
        - Tanca la finestra de configuració.
        """

        if self.habitacio:
            self.habitacio.temp_min_norm = self.temp_min_slider.value()
            self.habitacio.temp_max_norm = self.temp_max_slider.value()
            self.habitacio.hum_min_norm = self.hum_min_slider.value()
            self.habitacio.hum_max_norm = self.hum_max_slider.value()

            # Actualitzar la configuració global
            self.app_principal.config["habitacions"][self.habitacio_nom] = {
                "temp_min": self.habitacio.temp_min_norm,
                "temp_max": self.habitacio.temp_max_norm,
                "hum_min": self.habitacio.hum_min_norm,
                "hum_max": self.habitacio.hum_max_norm,
            }

            # Escriure la configuració al fitxer
            save_config(self.app_principal.config)

        # Actualitzar la pestanya de configuració general
        self.configuracio_pare.actualitzar_estat_habitacions()

        self.accept()  # Tancar la finestra