from PyQt5.QtWidgets import QWidget, QVBoxLayout, QHBoxLayout, QPushButton, QLabel, QGridLayout, QSpinBox, QScrollArea, QSlider
from PyQt5.QtCore import Qt
from configuracio.gestio_config import save_config, clear_layout
from components.config_habitacio import ConfigHabitacio

class Configuracio(QWidget):
    """
    Pantalla de configuració general de l'aplicació.
    Permet ajustar els valors normals de temperatura i humitat, així com
    gestionar el nombre d'habitacions monitoritzades. Inclou opcions per
    modificar configuracions individuals de cada habitació.
    """
    def __init__(self, app_principal):
        """
        Inicialitza la finestra de configuració i carrega l'aplicació principal.

        Paràmetres:
        ----------
        app_principal: Referència a l'objecte principal de l'aplicació.
        """

        super().__init__()
        self.app_principal = app_principal
        self.initUI()
    
    def guardar_configuracio(self):
        """
        Guarda la configuració actual (temperatura, humitat i nombre d'habitacions)
        al fitxer de configuració.
        """

        for habitacio_nom, habitacio in self.app_principal.habitacions.items():
            self.app_principal.config["habitacions"][habitacio_nom] = {
                "temp_min": habitacio.temp_min_norm,
                "temp_max": habitacio.temp_max_norm,
                "hum_min": habitacio.hum_min_norm,
                "hum_max": habitacio.hum_max_norm
            }
        # Guardar el nombre d'habitacions
        self.app_principal.config["num_habitacions"] = self.spin_habitacions.value()
        # Escriure la configuració al fitxer
        save_config(self.app_principal.config)
        print("Configuració guardada.")

    def initUI(self):
        """
        Configura la interfície gràfica de la finestra de configuració general,
        incloent sliders per ajustar valors globals i una llista d'habitacions.
        """

        # Layout principal
        layout = QVBoxLayout()
        layout.setContentsMargins(10, 10, 10, 10)
        layout.setSpacing(10)

        # 1. Botó de "Tornar"
        self.boto_retorn = QPushButton("← Tornar", self)
        self.boto_retorn.setStyleSheet("""
            QPushButton {
                background-color: #37B47E;
                color: white;
                border: none;
                padding: 10px 20px;
                border-radius: 15px;
                font-size: 16px;
                font-family: 'Eurostile', sans-serif;
                font-weight: bold;
            }
            QPushButton:hover {
                background-color: #2e9e6f;
            }
            QPushButton:pressed {
                background-color: #26865f;
            }
        """)
        self.boto_retorn.clicked.connect(self.tornar_a_principal)
        layout.addWidget(self.boto_retorn, alignment=Qt.AlignTop | Qt.AlignLeft)

        # 2. Títol de la finestra
        etiqueta_config_general = QLabel("Configuració General", self)
        etiqueta_config_general.setStyleSheet("""
            QLabel {
                color: #37B47E;  /* Color verd */
                font-family: 'Eurostile', sans-serif;
                font-size: 26px;
                font-weight: bold;
            }
        """)
        layout.addWidget(etiqueta_config_general, alignment=Qt.AlignCenter)

        # 3. Controls globals (sliders i selector del nombre d'habitacions)
        controls_layout = QGridLayout()  # Utilitzar QGridLayout per centrar elements
        controls_layout.setContentsMargins(0, 0, 500, 0)
        controls_layout.setSpacing(20)

        # Sliders
        self.temp_min_slider, temp_min_label = self.crear_slider("Min Temp", 10, 40, self.app_principal.config.get("temp_min", 20), "temp_min")
        self.temp_max_slider, temp_max_label = self.crear_slider("Max Temp", 10, 40, self.app_principal.config.get("temp_max", 30), "temp_max")
        self.hum_min_slider, hum_min_label = self.crear_slider("Min Hum", 10, 90, self.app_principal.config.get("hum_min", 30), "hum_min")
        self.hum_max_slider, hum_max_label = self.crear_slider("Max Hum", 10, 90, self.app_principal.config.get("hum_max", 60), "hum_max")

        controls_layout.addWidget(temp_min_label, 0, 0, alignment=Qt.AlignRight | Qt.AlignVCenter)  # Etiqueta alineada a la dreta
        controls_layout.addWidget(self.temp_min_slider, 0, 1, alignment=Qt.AlignLeft | Qt.AlignVCenter)  # Slider a prop de l'etiqueta

        controls_layout.addWidget(temp_max_label, 1, 0, alignment=Qt.AlignRight | Qt.AlignVCenter)
        controls_layout.addWidget(self.temp_max_slider, 1, 1, alignment=Qt.AlignLeft | Qt.AlignVCenter)

        controls_layout.addWidget(hum_min_label, 2, 0, alignment=Qt.AlignRight | Qt.AlignVCenter)
        controls_layout.addWidget(self.hum_min_slider, 2, 1, alignment=Qt.AlignLeft | Qt.AlignVCenter)

        controls_layout.addWidget(hum_max_label, 3, 0, alignment=Qt.AlignRight | Qt.AlignVCenter)
        controls_layout.addWidget(self.hum_max_slider, 3, 1, alignment=Qt.AlignLeft | Qt.AlignVCenter)



        # Nombre d'habitacions
        selector_layout = QHBoxLayout()
        etiqueta_nombre_habitacions = QLabel("                                                                                         Nombre d'habitacions:", self)
        etiqueta_nombre_habitacions.setStyleSheet("""
            QLabel {
                color: #37B47E;
                font-family: 'Eurostile', sans-serif;
                font-size: 19px;
                font-weight: bold;
            }
        """)
        self.spin_habitacions = QSpinBox(self)
        self.spin_habitacions.setRange(1, 9)
        self.spin_habitacions.setValue(len(self.app_principal.habitacions))
        self.spin_habitacions.valueChanged.connect(self.actualitzar_num_habitacions)
        selector_layout.addWidget(etiqueta_nombre_habitacions)
        selector_layout.addWidget(self.spin_habitacions)

        controls_layout.addLayout(selector_layout, 4, 0, 1, 2, alignment=Qt.AlignCenter)
        layout.addLayout(controls_layout)

        # 4. Llistat d'habitacions (amb scroll)
        scroll_area = QScrollArea(self)
        scroll_area.setWidgetResizable(True)
        scroll_widget = QWidget()
        self.habitacions_layout = QGridLayout(scroll_widget)
        scroll_area.setWidget(scroll_widget)
        layout.addWidget(scroll_area)

        # Configurar les barres de scroll perquè no es mostren
        scroll_area.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        scroll_area.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)

        # Ajustar el layout del scroll_area al contingut
        scroll_area.setStyleSheet("""
            QScrollArea {
                border: none; 
                background: transparent; 
            }
            QScrollBar:vertical, QScrollBar:horizontal {
                background: transparent; 
            }
        """)
        scroll_widget.setContentsMargins(10, 10, 10, 10)  # Marges dins del scroll


        # Inicialitzar l'estat de les habitacions
        self.actualitzar_estat_habitacions()

        # Establir el layout principal
        self.setLayout(layout)
        self.setWindowTitle("Configuració")
        self.showMaximized()


    def crear_slider(self, prefix, min_val, max_val, valor_actual, tipus):
        """
        Crea un control de slider amb una etiqueta per ajustar valors numèrics.

        Paràmetres:
        ----------
        prefix (str): Nom del valor (e.g., "Min Temp").
        min_val (int): Valor mínim del slider.
        max_val (int): Valor màxim del slider.
        valor_actual (int): Valor inicial del slider.
        tipus (str): Tipus de valor a modificar (e.g., "temp_min").

        Retorna:
        -------
        tuple: Slider i etiqueta associada.
        """

        layout = QHBoxLayout()
        slider = QSlider(Qt.Horizontal)
        slider.setRange(min_val, max_val)
        slider.setValue(valor_actual)
        slider.setFixedWidth(560) 

        label = QLabel(f"{prefix}: {slider.value()}")
        label.setAlignment(Qt.AlignLeft)  
        label.setFixedWidth(100)  
        label.setStyleSheet("""
        QLabel {
            font-size: 16px;  /* Igual que la configuració d'habitacions */
            font-family: 'Eurostile', sans-serif;
            color: #333333;
            border: none; 
            padding: 10px;  /* Espai intern */
        }
    """)

        def update_label(value):
            label.setText(f"{prefix}: {value}")
            self.app_principal.config[tipus] = value
            self.actualitzar_habitacions(tipus, value)  
            self.guardar_configuracio()

        slider.valueChanged.connect(update_label)
        slider.setStyleSheet("""
            QSlider::groove:horizontal {
                height: 8px;
                background: #d3d3d3;
                border-radius: 4px;
            }
            QSlider::handle:horizontal {
                background: #37B47E;
                border: 1px solid #5c5c5c;
                width: 16px;
                height: 16px;
                margin: -5px 0;
                border-radius: 8px;
            }
            QSlider::sub-page:horizontal {
                background: #37B47E;
                border-radius: 4px;
            }
            QSlider::add-page:horizontal {
                background: #d3d3d3;
                border-radius: 4px;
            }
        """)
        layout.addWidget(label)
        layout.addWidget(slider)
        return slider, label  

    def actualitzar_habitacions(self, tipus, valor):
        """
        Actualitza els valors globals per a totes les habitacions segons el tipus.

        Paràmetres:
        ----------
        tipus (str): Tipus de valor a modificar (e.g., "temp_min").
        valor (int): Nou valor a aplicar.
        """

        for habitacio in self.app_principal.habitacions.values():
            if tipus == 'temp_min':
                habitacio.temp_min_norm = valor
            elif tipus == 'temp_max':
                habitacio.temp_max_norm = valor
            elif tipus == 'hum_min':
                habitacio.hum_min_norm = valor
            elif tipus == 'hum_max':
                habitacio.hum_max_norm = valor
        self.actualitzar_estat_habitacions()  # Refrescar valors

    def tornar_a_principal(self):
        """ 
        Torna a la pantalla principal, guardant la configuració actual.
        """

        self.guardar_configuracio()
        self.app_principal.show()
        self.hide()

    def actualitzar_num_habitacions(self):
        """
        Modifica el nombre d'habitacions monitoritzades i actualitza
        la interfície i la configuració guardada.
        """

        num_habitacions = self.spin_habitacions.value()
        self.app_principal.config["num_habitacions"] = num_habitacions
        save_config(self.app_principal.config)  
        self.app_principal.actualitzar_habitacions(num_habitacions)
        self.actualitzar_estat_habitacions()

    def actualitzar_estat_habitacions(self):
        """
        Actualitza la llista d'habitacions a la pantalla de configuració,
        mostrant els valors actuals de temperatura i humitat.
        """

        # Buidar completament el layout d'habitacions
        while self.habitacions_layout.count():
            item = self.habitacions_layout.takeAt(0)
            if item.layout():
                clear_layout(item.layout())
                del item
            elif item.widget():
                item.widget().deleteLater()

        self.habitacions_layout.setContentsMargins(350, 10, 350, 10)  


        # Tornar a afegir cada habitació segons el nombre seleccionat
        num_habitacions = self.spin_habitacions.value()
        for i in range(1, num_habitacions + 1):
            habitacio_nom = f"Habitació {i}"

            # Recuperar habitació o inicialitzar-ne una si no existeix
            habitacio = self.app_principal.habitacions.get(habitacio_nom)

            # Crear un widget per a cada habitació
            habitacio_widget = QWidget()
            habitacio_layout = QHBoxLayout(habitacio_widget)
            habitacio_layout.setContentsMargins(10, 10, 10, 10)
            habitacio_layout.setSpacing(10)  # Separació interna del layout

            # Aplicar estil de requadre a cada habitació
            habitacio_widget.setStyleSheet("""
                QWidget {
                    border: 1px solid #d3d3d3;
                    border-radius: 5px;
                    background-color: #f9f9f9;
                }
            """)

            # Etiqueta de l'habitació
            etiqueta_habitacio = QLabel(habitacio_nom)
            etiqueta_habitacio.setAlignment(Qt.AlignCenter)  # Centrar el text
            etiqueta_habitacio.setStyleSheet("""
                QLabel {
                    font-size: 18px; 
                    font-family: 'Eurostile', sans-serif;
                    color: #37B47E; /* Verd */
                    border: none; 
                    padding: 10px; 
                    font-weight: bold; 
                }
            """)
            habitacio_layout.addWidget(etiqueta_habitacio, alignment=Qt.AlignCenter)

            # Mostrar els valors de temperatura i humitat
            if habitacio:
                # Min Temp
                etiqueta_min_temp = QLabel(f"Min Temp: {habitacio.temp_min_norm}ºC")
                etiqueta_min_temp.setAlignment(Qt.AlignCenter)  # Centrar el text
                etiqueta_min_temp.setStyleSheet("""
                    QLabel {
                        font-size: 16px;
                        font-family: 'Eurostile', sans-serif;
                        color: #333333;
                        background-color: none; 
                        border: none; 
                        padding: 10px; 
                    }
                """)
                habitacio_layout.addWidget(etiqueta_min_temp, alignment=Qt.AlignCenter)

                # Max Temp
                etiqueta_max_temp = QLabel(f"Max Temp: {habitacio.temp_max_norm}ºC")
                etiqueta_max_temp.setAlignment(Qt.AlignCenter)  # Centrar el text
                etiqueta_max_temp.setStyleSheet("""
                    QLabel {
                        font-size: 16px;
                        font-family: 'Eurostile', sans-serif;
                        color: #333333;
                        border: none; 
                        padding: 10px;
                    }
                """)
                habitacio_layout.addWidget(etiqueta_max_temp, alignment=Qt.AlignCenter)

                # Min Hum
                etiqueta_min_hum = QLabel(f"Min Hum: {habitacio.hum_min_norm}%")
                etiqueta_min_hum.setAlignment(Qt.AlignCenter)  # Centrar el text
                etiqueta_min_hum.setStyleSheet("""
                    QLabel {
                        font-size: 16px;
                        font-family: 'Eurostile', sans-serif;
                        color: #333333;
                        border: none; 
                        padding: 10px;
                    }
                """)
                habitacio_layout.addWidget(etiqueta_min_hum, alignment=Qt.AlignCenter)

                # Max Hum
                etiqueta_max_hum = QLabel(f"Max Hum: {habitacio.hum_max_norm}%")
                etiqueta_max_hum.setAlignment(Qt.AlignCenter)  # Centrar el text
                etiqueta_max_hum.setStyleSheet("""
                    QLabel {
                        font-size: 16px;
                        font-family: 'Eurostile', sans-serif;
                        color: #333333;
                        border: none; 
                        padding: 10px;
                    }
                """)
                habitacio_layout.addWidget(etiqueta_max_hum, alignment=Qt.AlignCenter)

            else:
                habitacio_layout.addWidget(QLabel("Min Temp: No disponible"))
                habitacio_layout.addWidget(QLabel("Max Temp: No disponible"))
                habitacio_layout.addWidget(QLabel("Min Hum: No disponible"))
                habitacio_layout.addWidget(QLabel("Max Hum: No disponible"))

            # Botó per modificar cada habitació
            boto_modificar = QPushButton("Modificar")
            boto_modificar.setFixedWidth(100)
            boto_modificar.setStyleSheet("""
                QPushButton {
                    background-color: #37B47E;
                    color: white;
                    padding: 8px;
                    font-family: 'Eurostile', sans-serif;
                    font-size: 14px;
                    font-weight: bold;
                    border-radius: 5px;
                }
                QPushButton:hover {
                    background-color: #2e9e6f;
                }
                QPushButton:pressed {
                    background-color: #26865f;
                }
            """)
            boto_modificar.clicked.connect(lambda _, h=habitacio_nom: self.obrir_configuracio_habitacio(h))
            habitacio_layout.addWidget(boto_modificar)

            # Afegir el widget d'habitació al layout principal
            self.habitacions_layout.addWidget(habitacio_widget, i - 1, 0)



    def obrir_configuracio_habitacio(self, habitacio_nom):
        """
        Obre una finestra de configuració individual per a una habitació específica.

        Paràmetres:
        ----------
        habitacio_nom (str): Nom de l'habitació a configurar.
        """

        self.ventana_config_habitacio = ConfigHabitacio(self.app_principal, habitacio_nom, self)
        self.ventana_config_habitacio.exec_()